import CustomStore from "devextreme/data/custom_store";

export const getQuery = async ({ select, options }) => {
  const { sort, searchValue, searchExpr, filter, take, skip } = options;
  if (searchExpr && searchValue) {
    filter = filter
      ? [filter, "and", [searchExpr, "contains", searchValue]]
      : [searchExpr, "contains", searchValue];
  }

  let req = "";
  let first = true;
  if (select) {
    req += `${first ? "?" : "&"}select=${JSON.stringify(select)}`;
    first = false;
  }
  if (filter) {
    req += `${first ? "?" : "&"}filter=${JSON.stringify(filter)}`;
    first = false;
  }

  if (take !== undefined && skip !== undefined) {
    req += `${first ? "?" : "&"}take=${Number(take)}&skip=${Number(skip)}`;
    first = false;
  }
  if (sort) {
    req += `${first ? "?" : "&"}sort=${JSON.stringify(sort)}`;
    first = false;
  }
  return req;
};

export const getCustomDataSource = ({ key, load, byKey }) => {
  return new CustomStore({
    key: key || "id",
    load: async (options) => {
      return await load(options);
    },
    byKey: byKey
      ? async (id) => {
          return await byKey(id);
        }
      : undefined,
  });
};

export const getLookupDataStore = ({ key, load }) => {
  return getCustomDataSource({
    key,
    load: load,
    byKey: async (id) => {
      const { data, totalCount } = await load({
        filter: [key, "=", id],
      });
      return data && data.length > 0 ? data[0] : null;
    },
  });
};
