const operators = {
  "=": "_eq",
  contains: "_ilike",
  notcontains: "_nilike",
  startswith: "_ilike",
  endswith: "_ilike",
  "<>": "_neq",
  "<": "_lt",
  ">": "_gt",
  ">=": "_gte",
  "<=": "_lte",
};

const getOperator = (opString) => {
  if (operators[opString]) {
    return operators[opString];
  } else {
    return null;
  }
};

const getFilterValue = (opString, filterValue) => {
  if (opString == "contains") {
    return "%" + filterValue + "%";
  }
  if (opString == "notcontains") {
    return "%" + filterValue + "%";
  }
  if (opString == "startswith") {
    return "" + filterValue + "%";
  }
  if (opString == "endswith") {
    return "%" + filterValue + "";
  }
  return filterValue;
};

const getFieldName = (fieldName) => {
  return fieldName.split(".");
};

const getFilter = (query, filter) => {
  if (filter == null) {
    return;
  }
  if (Array.isArray(filter[0]) == false && filter.length == 3) {
    let fieldName = getFieldName(filter[0]);
    let operator = getOperator(filter[1]);
    let filterValue = getFilterValue(filter[1], filter[2]);
    if (operator != null) {
      let Q = query;
      fieldName.forEach((fn, index) => {
        if (fieldName.length - 1 === index) {
          Q[fn] = { [operator]: filterValue };
        } else {
          Q[fn] = {};
          Q = Q[fn];
        }
      });
    }
  } else if (Array.isArray(filter[0])) {
    let q = {};
    if (filter[1] === "and") {
      query["_and"] = [];
      q = query["_and"];
    } else {
      query["_or"] = [];
      q = query["_or"];
    }
    let index = 0;
    for (let i = 0; i < filter.length; i += 2) {
      q[index] = {};
      getFilter(q[index], filter[i]);
      index++;
    }
  }
  return;
};

const getFilterQuery = (filterParam) => {
  let filter = { where: {} };
  getFilter(filter.where, filterParam);
  if (Object.keys(filter.where).length === 0) {
    return "";
  }
  filter = JSON.stringify(filter);
  filter = filter.substring(1, filter.length - 1);
  filter = filter.replace(/"([^"]+)":/g, "$1:");
  return filter;
};

const replaceChar = (result) => {
  result = result.replace(/"([^"]+)":/g, "$1:");
  return result;
};

const getSort = (sort) => {
  let result = "";
  let list = [];
  for (let i = 0; i < sort.length; i++) {
    list.push(`{ selector: "${sort[i].selector}" ,desc:${sort[i].desc} }`);
  }
  result = `sort: [${list.join(",")}]`;
  return result;
};

const getTakeSkipQuery = ({ take, skip }) => {
  let result = {};
  try {
    if (take) {
      result.limit = parseInt(take);
    } else {
      result.limit = 20;
    }
    if (skip) {
      result.offset = parseInt(skip);
    } else {
      result.offset = 0;
    }
  } catch (ex) {
    return " page:{ limit: 20, offset: 0 }";
  }
  result = ` page: ${JSON.stringify(result)}`;
  result = result.replace(/"/g, "");
  return result;
};

export const list = async ({ objectName, options, fields, first }) => {
  const sort = options.sort ? getSort(options.sort) : "";

  const searchValue = options.searchValue;
  const searchExpr = options.searchExpr;

  if (searchExpr && searchValue) {
    options.filter = options.filter
      ? [options.filter, "and", [searchExpr, "contains", searchValue]]
      : [searchExpr, "contains", searchValue];
  }

  const filter = getFilterQuery(options.filter);
  const takeSkip = getTakeSkipQuery({
    take: options.take,
    skip: options.skip,
  });

  const query = `query listQuery {
            ${objectName} (${[sort, filter, takeSkip]
    .filter((s) => s)
    .join(",")}) {
    rows 
    { 
        ${fields}
    }
    count
    }
    }`;
  console.log(query);

  return query;
  //   const response = await call({
  //     query: query,
  //   });
  //   const data = response.data;
  //   if (first) {
  //     return {
  //       status: "success",
  //       data: data[objectName].rows.length > 0 ? data[objectName].rows[0] : null,
  //     };
  //   }
  //   return {
  //     status: "success",
  //     data: data[objectName].rows,
  //     totalCount: data[objectName].count,
  //   };
};

export const one = async ({ objectName, id, fields }) => {
  return `query oneQuery {
        ${objectName}_one (id:"${id}") { ${fields} }
      }`;

  //   let responseData = response.data;
  //   let errors = responseData.errors;

  //   if (errors && errors.length > 0) {
  //     return {
  //       status: "failed",
  //       message: errors.map((s) => s.message + " "),
  //     };
  //   }
  //   if (responseData[objectName + "_one"]) {
  //     return {
  //       status: "success",
  //       data: responseData[objectName + "_one"],
  //     };
  //   }
};

export const update = async ({ objectName, id, data, fields }) => {
  data = JSON.stringify(data);
  data = replaceChar(data);

  return `mutation updateQuery {
        update_${objectName} (id: "${id}" data:${data}) { ${fields} }
    }`;

  //   const responseData = response.data;
  //   const errors = responseData.errors;

  //   if (errors && errors.length > 0) {
  //     console.log(errors.map((s) => s.message + ""));
  //     return {
  //       status: "failed",
  //       message: errors.map((s) => s.message + ""),
  //       data: null,
  //     };
  //   }
  //   if (responseData[`update_${objectName}`]) {
  //     return {
  //       status: "success",
  //       message: "Амжилттай хадгаллаа",
  //       data: responseData[`update_${objectName}`],
  //     };
  //   }
};

export const create = async ({ objectName, data, fields }) => {
  data = JSON.stringify(data);
  data = replaceChar(data);
  return `mutation createQuery {
        create_${objectName} (data:${data}) { ${fields} }
    }`;

  //   const responseData = response.data;
  //   const errors = responseData.errors;

  //   if (errors && errors.length > 0) {
  //     console.log(errors.map((s) => s.message + ""));
  //     return {
  //       status: "failed",
  //       message: errors.map((s) => s.message + ""),
  //       data: null,
  //     };
  //   }
  //   if (responseData[`create_${objectName}`]) {
  //     return {
  //       status: "success",
  //       message: "Амжилттай хадгаллаа",
  //       data: responseData[`create_${objectName}`],
  //     };
  //   }
};
